@@ -11,17 +11,17 @@
 # under the License.
 
 import datetime
+import io
 import itertools
 import json
 import logging
 import sys
+from unittest import mock
 import uuid
 
-import mock
 from oslo_utils import encodeutils
 import requests
 import requests.auth
-import six
 from testtools import matchers
 
 from keystoneauth1 import adapter
@@ -164,6 +164,17 @@ class SessionTests(utils.TestCase):
         self.assertEqual(headers['OpenStack-API-Version'], 'volume 2.30')
         self.assertEqual(len(headers.keys()), 1)
 
+        # shared file system service-type - shared-file-system microversion
+        # (with service type aliases)
+        for service_type in ['sharev2', 'shared-file-system']:
+            headers = {}
+            client_session.Session._set_microversion_headers(
+                headers, (2, 30), None, {'service_type': service_type})
+            self.assertEqual(headers['X-OpenStack-Manila-API-Version'], '2.30')
+            self.assertEqual(headers['OpenStack-API-Version'],
+                             'shared-file-system 2.30')
+            self.assertEqual(len(headers.keys()), 2)
+
         # Headers already exist - no change
         headers = {
             'OpenStack-API-Version': 'compute 2.30',
@@ -372,11 +383,13 @@ class SessionTests(utils.TestCase):
         """Verify response body is only logged in specific content types.
 
         Response bodies are logged only when the response's Content-Type header
-        is set to application/json. This prevents us to get an unexpected
-        MemoryError when reading arbitrary responses, such as streams.
+        is set to application/json or text/plain. This prevents us to get an
+        unexpected MemoryError when reading arbitrary responses, such as
+        streams.
         """
-        OMITTED_BODY = ('Omitted, Content-Type is set to %s. Only '
-                        'application/json responses have their bodies logged.')
+        OMITTED_BODY = ('Omitted, Content-Type is set to %s. Only ' +
+                        ', '.join(client_session._LOG_CONTENT_TYPES) +
+                        ' responses have their bodies logged.')
         session = client_session.Session(verify=False)
 
         # Content-Type is not set
@@ -411,6 +424,23 @@ class SessionTests(utils.TestCase):
         self.assertNotIn(OMITTED_BODY % 'application/json; charset=UTF-8',
                          self.logger.output)
 
+        # Content-Type is set to text/plain
+        text = 'Error detected, unable to continue.'
+        self.stub_url('POST', text=text,
+                      headers={'Content-Type': 'text/plain'})
+        session.post(self.TEST_URL)
+        self.assertIn(text, self.logger.output)
+        self.assertNotIn(OMITTED_BODY % 'text/plain', self.logger.output)
+
+        # Content-Type is set to text/plain; charset=UTF-8
+        text = 'Error detected, unable to continue.'
+        self.stub_url('POST', text=text,
+                      headers={'Content-Type': 'text/plain; charset=UTF-8'})
+        session.post(self.TEST_URL)
+        self.assertIn(text, self.logger.output)
+        self.assertNotIn(OMITTED_BODY % 'text/plain; charset=UTF-8',
+                         self.logger.output)
+
     def test_logging_cacerts(self):
         path_to_certs = '/path/to/certs'
         session = client_session.Session(verify=path_to_certs)
@@ -421,24 +451,66 @@ class SessionTests(utils.TestCase):
         self.assertIn('--cacert', self.logger.output)
         self.assertIn(path_to_certs, self.logger.output)
 
-    def test_connect_retries(self):
+    def _connect_retries_check(self, session, expected_retries=0,
+                               call_args=None):
+        call_args = call_args or {}
+
         self.stub_url('GET', exc=requests.exceptions.Timeout())
 
-        session = client_session.Session()
-        retries = 3
+        call_args['url'] = self.TEST_URL
 
         with mock.patch('time.sleep') as m:
             self.assertRaises(exceptions.ConnectTimeout,
                               session.get,
-                              self.TEST_URL, connect_retries=retries)
+                              **call_args)
 
-            self.assertEqual(retries, m.call_count)
+            self.assertEqual(expected_retries, m.call_count)
             # 3 retries finishing with 2.0 means 0.5, 1.0 and 2.0
             m.assert_called_with(2.0)
 
         # we count retries so there will be one initial request + 3 retries
         self.assertThat(self.requests_mock.request_history,
-                        matchers.HasLength(retries + 1))
+                        matchers.HasLength(expected_retries + 1))
+
+    def test_session_connect_retries(self):
+        retries = 3
+        session = client_session.Session(connect_retries=retries)
+        self._connect_retries_check(session=session, expected_retries=retries)
+
+    def test_call_args_connect_retries_session_init(self):
+        session = client_session.Session()
+        retries = 3
+        call_args = {'connect_retries': retries}
+        self._connect_retries_check(session=session,
+                                    expected_retries=retries,
+                                    call_args=call_args)
+
+    def test_call_args_connect_retries_overrides_session_retries(self):
+        session_retries = 6
+        call_arg_retries = 3
+        call_args = {'connect_retries': call_arg_retries}
+        session = client_session.Session(connect_retries=session_retries)
+        self._connect_retries_check(session=session,
+                                    expected_retries=call_arg_retries,
+                                    call_args=call_args)
+
+    def test_override_session_connect_retries_for_request(self):
+        session_retries = 1
+        session = client_session.Session(connect_retries=session_retries)
+
+        self.stub_url('GET', exc=requests.exceptions.Timeout())
+        call_args = {'connect_retries': 0}
+
+        with mock.patch('time.sleep') as m:
+            self.assertRaises(
+                exceptions.ConnectTimeout,
+                session.request,
+                self.TEST_URL,
+                'GET',
+                **call_args
+            )
+
+            self.assertEqual(0, m.call_count)
 
     def test_connect_retries_interval_limit(self):
         self.stub_url('GET', exc=requests.exceptions.Timeout())
@@ -936,7 +1008,7 @@ class SessionAuthTests(utils.TestCase):
         mock_close = mock.Mock()
         sess._session.close = mock_close
         del sess
-        mock_close.assert_called_once()
+        self.assertEqual(1, mock_close.call_count)
 
     def test_service_type_urls(self):
         service_type = 'compute'
@@ -1186,8 +1258,8 @@ class SessionAuthTests(utils.TestCase):
         logger.setLevel(logging.DEBUG)
         logger.propagate = False
 
-        io = six.StringIO()
-        handler = logging.StreamHandler(io)
+        string_io = io.StringIO()
+        handler = logging.StreamHandler(string_io)
         logger.addHandler(handler)
 
         auth = AuthPlugin()
@@ -1201,7 +1273,7 @@ class SessionAuthTests(utils.TestCase):
         resp = sess.get(self.TEST_URL, logger=logger)
 
         self.assertEqual(response, resp.json())
-        output = io.getvalue()
+        output = string_io.getvalue()
 
         self.assertIn(self.TEST_URL, output)
         self.assertIn(list(response.keys())[0], output)
@@ -1217,14 +1289,14 @@ class SessionAuthTests(utils.TestCase):
             logger = logging.getLogger(logger_name)
             logger.setLevel(logging.DEBUG)
 
-            io = six.StringIO()
-            handler = logging.StreamHandler(io)
+            string_io = io.StringIO()
+            handler = logging.StreamHandler(string_io)
             logger.addHandler(handler)
-            return io
+            return string_io
 
-        io = {}
+        io_dict = {}
         for name in ('request', 'body', 'response', 'request-id'):
-            io[name] = get_logger_io(name)
+            io_dict[name] = get_logger_io(name)
 
         auth = AuthPlugin()
         sess = client_session.Session(auth=auth, split_loggers=True)
@@ -1250,10 +1322,10 @@ class SessionAuthTests(utils.TestCase):
 
         self.assertEqual(response, resp.json())
 
-        request_output = io['request'].getvalue().strip()
-        response_output = io['response'].getvalue().strip()
-        body_output = io['body'].getvalue().strip()
-        id_output = io['request-id'].getvalue().strip()
+        request_output = io_dict['request'].getvalue().strip()
+        response_output = io_dict['response'].getvalue().strip()
+        body_output = io_dict['body'].getvalue().strip()
+        id_output = io_dict['request-id'].getvalue().strip()
 
         self.assertIn('curl -g -i -X GET {url}'.format(url=self.TEST_URL),
                       request_output)
@@ -1535,8 +1607,8 @@ class AdapterTest(utils.TestCase):
         logger.setLevel(logging.DEBUG)
         logger.propagate = False
 
-        io = six.StringIO()
-        handler = logging.StreamHandler(io)
+        string_io = io.StringIO()
+        handler = logging.StreamHandler(string_io)
         logger.addHandler(handler)
 
         auth = AuthPlugin()
@@ -1551,7 +1623,7 @@ class AdapterTest(utils.TestCase):
         resp = adpt.get(self.TEST_URL, logger=logger)
 
         self.assertEqual(response, resp.json())
-        output = io.getvalue()
+        output = string_io.getvalue()
 
         self.assertIn(self.TEST_URL, output)
         self.assertIn(list(response.keys())[0], output)
@@ -1780,7 +1852,7 @@ class AdapterTest(utils.TestCase):
             with mock.patch.object(sess, 'request') as m:
                 adapter.Adapter(sess, **adap_kwargs).get(url, **get_kwargs)
                 m.assert_called_once_with(url, 'GET', endpoint_filter={},
-                                          rate_semaphore=mock.ANY,
+                                          headers={}, rate_semaphore=mock.ANY,
                                           **exp_kwargs)
 
         # No default_microversion in Adapter, no microversion in get()
@@ -1804,7 +1876,7 @@ class AdapterTest(utils.TestCase):
             with mock.patch.object(sess, 'request') as m:
                 adapter.Adapter(sess, **adap_kwargs).get(url, **get_kwargs)
                 m.assert_called_once_with(url, 'GET', endpoint_filter={},
-                                          rate_semaphore=mock.ANY,
+                                          headers={}, rate_semaphore=mock.ANY,
                                           **exp_kwargs)
 
         # No raise_exc in Adapter or get()
